@@ -30,7 +30,17 @@
 #include <errno.h>
 #include "rdesktop.h"
 
+#ifdef HAVE_ICONV
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+#ifdef HAVE_LANGINFO_H
+#include <langinfo.h>
+#endif
+#endif
+
 #ifdef EGD_SOCKET
+#include <sys/types.h>
 #include <sys/socket.h>		/* socket connect */
 #include <sys/un.h>		/* sockaddr_un */
 #endif
@@ -51,6 +61,10 @@ int g_width = 800;		/* width is special: If 0, the
 int g_height = 600;
 int g_xpos = 0;
 int g_ypos = 0;
+int g_pos = 0;			/* 0 position unspecified,
+				   1 specified,
+				   2 xpos neg,
+				   4 ypos neg  */
 extern int g_tcp_port_rdp;
 int g_server_bpp = 8;
 int g_win_button_size = 0;	/* If zero, disable single app mode */
@@ -61,7 +75,8 @@ BOOL g_bitmap_cache_persist_enable = False;
 BOOL g_bitmap_cache_precache = True;
 BOOL g_encryption = True;
 BOOL packet_encryption = True;
-BOOL g_desktop_save = True;
+BOOL g_desktop_save = True;	/* desktop save order */
+BOOL g_polygon_ellipse_orders = True;	/* polygon / ellipse orders */
 BOOL g_fullscreen = False;
 BOOL g_grab_keyboard = True;
 BOOL g_hide_decorations = False;
@@ -78,6 +93,10 @@ uint32 g_rdp5_performanceflags =
 BOOL g_rdpsnd = False;
 #endif
 
+#ifdef HAVE_ICONV
+char g_codepage[16] = "";
+#endif
+
 extern RDPDR_DEVICE g_rdpdr_device[];
 extern uint32 g_num_devices;
 extern char *g_rdpdr_clientname;
@@ -112,6 +131,9 @@ usage(char *program)
 	fprintf(stderr, "   -g: desktop geometry (WxH)\n");
 	fprintf(stderr, "   -f: full-screen mode\n");
 	fprintf(stderr, "   -b: force bitmap updates\n");
+#ifdef HAVE_ICONV
+	fprintf(stderr, "   -L: local codepage\n");
+#endif
 	fprintf(stderr, "   -B: use BackingStore of X-server (if available)\n");
 	fprintf(stderr, "   -e: disable encryption (French TS)\n");
 	fprintf(stderr, "   -E: disable encryption from client to server\n");
@@ -366,7 +388,7 @@ main(int argc, char *argv[])
 #endif
 
 	while ((c = getopt(argc, argv,
-			   VNCOPT "u:d:s:c:p:n:k:g:fbBeEmzCDKS:T:NX:a:x:Pr:045h?")) != -1)
+			   VNCOPT "u:L:d:s:c:p:n:k:g:fbBeEmzCDKS:T:NX:a:x:Pr:045h?")) != -1)
 	{
 		switch (c)
 		{
@@ -389,6 +411,14 @@ main(int argc, char *argv[])
 				username_option = 1;
 				break;
 
+			case 'L':
+#ifdef HAVE_ICONV
+				STRNCPY(g_codepage, optarg, sizeof(g_codepage));
+#else
+				error("iconv support not available\n");
+#endif
+				break;
+
 			case 'd':
 				STRNCPY(domain, optarg, sizeof(domain));
 				break;
@@ -456,10 +486,16 @@ main(int argc, char *argv[])
 				}
 
 				if (*p == '+' || *p == '-')
+				{
+					g_pos |= (*p == '-') ? 2 : 1;
 					g_xpos = strtol(p, &p, 10);
 
+				}
 				if (*p == '+' || *p == '-')
+				{
+					g_pos |= (*p == '-') ? 4 : 1;
 					g_ypos = strtol(p, NULL, 10);
+				}
 
 				break;
 
@@ -523,7 +559,7 @@ main(int argc, char *argv[])
 				break;
 
 			case 'X':
-				g_embed_wnd = strtol(optarg, NULL, 10);
+				g_embed_wnd = strtol(optarg, NULL, 0);
 				break;
 
 			case 'a':
@@ -538,7 +574,7 @@ main(int argc, char *argv[])
 
 			case 'z':
 				DEBUG(("rdp compression enabled\n"));
-				flags |= RDP_COMPRESSION;
+				flags |= (RDP_LOGON_COMPRESSION | RDP_LOGON_COMPRESSION2);
 				break;
 
 			case 'x':
@@ -675,6 +711,20 @@ main(int argc, char *argv[])
 		STRNCPY(g_username, pw->pw_name, sizeof(g_username));
 	}
 
+#ifdef HAVE_ICONV
+	if (g_codepage[0] == 0)
+	{
+		if (setlocale(LC_CTYPE, ""))
+		{
+			STRNCPY(g_codepage, nl_langinfo(CODESET), sizeof(g_codepage));
+		}
+		else
+		{
+			STRNCPY(g_codepage, DEFAULT_CODEPAGE, sizeof(g_codepage));
+		}
+	}
+#endif
+
 	if (g_hostname[0] == 0)
 	{
 		if (gethostname(fullhostname, sizeof(fullhostname)) == -1)
@@ -690,12 +740,6 @@ main(int argc, char *argv[])
 		STRNCPY(g_hostname, fullhostname, sizeof(g_hostname));
 	}
 
-	if ((flags & RDP_COMPRESSION) && (g_server_bpp > 8))
-	{
-		warning("rdp compression not supported for bpp > 8, compression disabled\n");
-		flags ^= RDP_COMPRESSION;
-	}
-
 	if (prompt_password && read_password(password, sizeof(password)))
 		flags |= RDP_LOGON_AUTO;
 
