@@ -155,7 +155,7 @@ add_async_iorequest(uint32 device, uint32 file, uint32 id, uint32 major, uint32
 
 	while (iorq->fd != 0)
 	{
-		// create new element if needed
+		/* create new element if needed */
 		if (iorq->next == NULL)
 		{
 			iorq->next =
@@ -228,7 +228,7 @@ announcedata_size()
 	int size, i;
 	PRINTER *printerinfo;
 
-	size = 8;		//static announce size
+	size = 8;		/* static announce size */
 	size += g_num_devices * 0x14;
 
 	for (i = 0; i < g_num_devices; i++)
@@ -327,7 +327,7 @@ rdpdr_send_completion(uint32 device, uint32 id, uint32 status, uint32 result, ui
 	/* JIF */
 #ifdef WITH_DEBUG_RDP5
 	printf("--> rdpdr_send_completion\n");
-	//hexdump(s->channel_hdr + 8, s->end - s->channel_hdr - 8);
+	/* hexdump(s->channel_hdr + 8, s->end - s->channel_hdr - 8); */
 #endif
 	channel_send(s, rdpdr_channel);
 }
@@ -406,7 +406,7 @@ rdpdr_process_irp(STREAM s)
 		case IRP_MJ_CREATE:
 
 			in_uint32_be(s, desired_access);
-			in_uint8s(s, 0x08);	// unknown
+			in_uint8s(s, 0x08);	/* unknown */
 			in_uint32_le(s, error_mode);
 			in_uint32_le(s, share_mode);
 			in_uint32_le(s, disposition);
@@ -463,7 +463,7 @@ rdpdr_process_irp(STREAM s)
 				break;
 			}
 
-			if (rw_blocking)	// Complete read immediately
+			if (rw_blocking)	/* Complete read immediately */
 			{
 				buffer = (uint8 *) xrealloc((void *) buffer, length);
 				if (!buffer)
@@ -476,7 +476,7 @@ rdpdr_process_irp(STREAM s)
 				break;
 			}
 
-			// Add request to table
+			/* Add request to table */
 			pst_buf = (uint8 *) xmalloc(length);
 			if (!pst_buf)
 			{
@@ -516,13 +516,13 @@ rdpdr_process_irp(STREAM s)
 				break;
 			}
 
-			if (rw_blocking)	// Complete immediately
+			if (rw_blocking)	/* Complete immediately */
 			{
 				status = fns->write(file, s->p, length, offset, &result);
 				break;
 			}
 
-			// Add to table
+			/* Add to table */
 			pst_buf = (uint8 *) xmalloc(length);
 			if (!pst_buf)
 			{
@@ -629,7 +629,7 @@ rdpdr_process_irp(STREAM s)
 					/* JIF
 					   unimpl("IRP major=0x%x minor=0x%x: IRP_MN_NOTIFY_CHANGE_DIRECTORY\n", major, minor);  */
 
-					in_uint32_le(s, info_level);	// notify mask
+					in_uint32_le(s, info_level);	/* notify mask */
 
 					g_notify_stamp = True;
 
@@ -835,7 +835,7 @@ rdpdr_init()
 void
 rdpdr_add_fds(int *n, fd_set * rfds, fd_set * wfds, struct timeval *tv, BOOL * timeout)
 {
-	uint32 select_timeout = 0;	// Timeout value to be used for select() (in millisecons).
+	uint32 select_timeout = 0;	/* Timeout value to be used for select() (in millisecons). */
 	struct async_iorequest *iorq;
 	char c;
 
@@ -920,7 +920,7 @@ rdpdr_remove_iorequest(struct async_iorequest *prev, struct async_iorequest *ior
 	}
 	else
 	{
-		// Even if NULL
+		/* Even if NULL */
 		g_iorequest = iorq->next;
 		xfree(iorq);
 		iorq = NULL;
@@ -959,7 +959,7 @@ _rdpdr_check_fds(fd_set * rfds, fd_set * wfds, BOOL timed_out)
 				{
 
 					/* iv_timeout between 2 chars, send partial_len */
-					//printf("RDPDR: IVT total %u bytes read of %u\n", iorq->partial_len, iorq->length);
+					/*printf("RDPDR: IVT total %u bytes read of %u\n", iorq->partial_len, iorq->length); */
 					rdpdr_send_completion(iorq->device,
 							      iorq->id, STATUS_SUCCESS,
 							      iorq->partial_len,
@@ -1180,8 +1180,8 @@ rdpdr_abort_io(uint32 fd, uint32 major, NTSTATUS status)
 	prev = NULL;
 	while (iorq != NULL)
 	{
-		// Only remove from table when major is not set, or when correct major is supplied.
-		// Abort read should not abort a write io request.
+		/* Only remove from table when major is not set, or when correct major is supplied.
+		   Abort read should not abort a write io request. */
 		if ((iorq->fd == fd) && (major == 0 || iorq->major == major))
 		{
 			result = 0;
