@@ -32,6 +32,7 @@ extern int g_width;
 extern int g_height;
 extern int g_xpos;
 extern int g_ypos;
+extern int g_pos;
 extern BOOL g_sendmotion;
 extern BOOL g_fullscreen;
 extern BOOL g_grab_keyboard;
@@ -119,6 +120,31 @@ PixelColour;
 	XFillRectangle(g_display, g_ownbackstore ? g_backstore : g_wnd, g_gc, x, y, cx, cy); \
 }
 
+#define FILL_POLYGON(p,np)\
+{ \
+	XFillPolygon(g_display, g_wnd, g_gc, p, np, Complex, CoordModePrevious); \
+	if (g_ownbackstore) \
+		XFillPolygon(g_display, g_backstore, g_gc, p, np, Complex, CoordModePrevious); \
+}
+
+#define DRAW_ELLIPSE(x,y,cx,cy,m)\
+{ \
+	switch (m) \
+	{ \
+		case 0:	/* Outline */ \
+			XDrawArc(g_display, g_wnd, g_gc, x, y, cx, cy, 0, 360*64); \
+			if (g_ownbackstore) \
+				XDrawArc(g_display, g_backstore, g_gc, x, y, cx, cy, 0, 360*64); \
+			break; \
+		case 1: /* Filled */ \
+			XFillArc(g_display, g_ownbackstore ? g_backstore : g_wnd, g_gc, x, y, \
+				 cx, cy, 0, 360*64); \
+			if (g_ownbackstore) \
+				XCopyArea(g_display, g_backstore, g_wnd, g_gc, x, y, cx, cy, x, y); \
+			break; \
+	} \
+}
+
 /* colour maps */
 extern BOOL g_owncolmap;
 static Colormap g_xcolmap;
@@ -1138,6 +1164,12 @@ ui_create_window(void)
 	wndwidth = g_fullscreen ? WidthOfScreen(g_screen) : g_width;
 	wndheight = g_fullscreen ? HeightOfScreen(g_screen) : g_height;
 
+	/* Handle -x-y portion of geometry string */
+	if (g_xpos < 0 || (g_xpos == 0 && (g_pos & 2)))
+		g_xpos = WidthOfScreen(g_screen) + g_xpos - g_width;
+	if (g_ypos < 0 || (g_ypos == 0 && (g_pos & 4)))
+		g_ypos = HeightOfScreen(g_screen) + g_ypos - g_height;
+
 	attribs.background_pixel = BlackPixelOfScreen(g_screen);
 	attribs.border_pixel = WhitePixelOfScreen(g_screen);
 	attribs.backing_store = g_ownbackstore ? NotUseful : Always;
@@ -1181,6 +1213,8 @@ ui_create_window(void)
 	if (sizehints)
 	{
 		sizehints->flags = PMinSize | PMaxSize;
+		if (g_pos)
+			sizehints->flags |= PPosition;
 		sizehints->min_width = sizehints->max_width = g_width;
 		sizehints->min_height = sizehints->max_height = g_height;
 		XSetWMNormalHints(g_display, g_wnd, sizehints);
@@ -1307,7 +1341,7 @@ xwin_toggle_fullscreen(void)
 	}
 }
 
-/* Process all events in Xlib queue
+/* Process events in Xlib queue
    Returns 0 after user quit, 1 otherwise */
 static int
 xwin_process_events(void)
@@ -1319,8 +1353,9 @@ xwin_process_events(void)
 	key_translation tr;
 	char str[256];
 	Status status;
+	int events = 0;
 
-	while (XPending(g_display) > 0)
+	while ((XPending(g_display) > 0) && events++ < 20)
 	{
 		XNextEvent(g_display, &xevent);
 
@@ -2166,6 +2201,151 @@ ui_rect(
 	FILL_RECTANGLE(x, y, cx, cy);
 }
 
+void
+ui_polygon(uint8 opcode,
+	   /* mode */ uint8 fillmode,
+	   /* dest */ POINT * point, int npoints,
+	   /* brush */ BRUSH * brush, int bgcolour, int fgcolour)
+{
+	uint8 style, i, ipattern[8];
+	Pixmap fill;
+
+	SET_FUNCTION(opcode);
+
+	switch (fillmode)
+	{
+		case ALTERNATE:
+			XSetFillRule(g_display, g_gc, EvenOddRule);
+			break;
+		case WINDING:
+			XSetFillRule(g_display, g_gc, WindingRule);
+			break;
+		default:
+			unimpl("fill mode %d\n", fillmode);
+	}
+
+	if (brush)
+		style = brush->style;
+	else
+		style = 0;
+
+	switch (style)
+	{
+		case 0:	/* Solid */
+			SET_FOREGROUND(fgcolour);
+			FILL_POLYGON((XPoint *) point, npoints);
+			break;
+
+		case 2:	/* Hatch */
+			fill = (Pixmap) ui_create_glyph(8, 8,
+							hatch_patterns + brush->pattern[0] * 8);
+			SET_FOREGROUND(fgcolour);
+			SET_BACKGROUND(bgcolour);
+			XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
+			XSetStipple(g_display, g_gc, fill);
+			XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
+			FILL_POLYGON((XPoint *) point, npoints);
+			XSetFillStyle(g_display, g_gc, FillSolid);
+			XSetTSOrigin(g_display, g_gc, 0, 0);
+			ui_destroy_glyph((HGLYPH) fill);
+			break;
+
+		case 3:	/* Pattern */
+			for (i = 0; i != 8; i++)
+				ipattern[7 - i] = brush->pattern[i];
+			fill = (Pixmap) ui_create_glyph(8, 8, ipattern);
+			SET_FOREGROUND(bgcolour);
+			SET_BACKGROUND(fgcolour);
+			XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
+			XSetStipple(g_display, g_gc, fill);
+			XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
+			FILL_POLYGON((XPoint *) point, npoints);
+			XSetFillStyle(g_display, g_gc, FillSolid);
+			XSetTSOrigin(g_display, g_gc, 0, 0);
+			ui_destroy_glyph((HGLYPH) fill);
+			break;
+
+		default:
+			unimpl("brush %d\n", brush->style);
+	}
+
+	RESET_FUNCTION(opcode);
+}
+
+void
+ui_polyline(uint8 opcode,
+	    /* dest */ POINT * points, int npoints,
+	    /* pen */ PEN * pen)
+{
+	/* TODO: set join style */
+	SET_FUNCTION(opcode);
+	SET_FOREGROUND(pen->colour);
+	XDrawLines(g_display, g_wnd, g_gc, (XPoint *) points, npoints, CoordModePrevious);
+	if (g_ownbackstore)
+		XDrawLines(g_display, g_backstore, g_gc, (XPoint *) points, npoints,
+			   CoordModePrevious);
+	RESET_FUNCTION(opcode);
+}
+
+void
+ui_ellipse(uint8 opcode,
+	   /* mode */ uint8 fillmode,
+	   /* dest */ int x, int y, int cx, int cy,
+	   /* brush */ BRUSH * brush, int bgcolour, int fgcolour)
+{
+	uint8 style, i, ipattern[8];
+	Pixmap fill;
+
+	SET_FUNCTION(opcode);
+
+	if (brush)
+		style = brush->style;
+	else
+		style = 0;
+
+	switch (style)
+	{
+		case 0:	/* Solid */
+			SET_FOREGROUND(fgcolour);
+			DRAW_ELLIPSE(x, y, cx, cy, fillmode);
+			break;
+
+		case 2:	/* Hatch */
+			fill = (Pixmap) ui_create_glyph(8, 8,
+							hatch_patterns + brush->pattern[0] * 8);
+			SET_FOREGROUND(fgcolour);
+			SET_BACKGROUND(bgcolour);
+			XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
+			XSetStipple(g_display, g_gc, fill);
+			XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
+			DRAW_ELLIPSE(x, y, cx, cy, fillmode);
+			XSetFillStyle(g_display, g_gc, FillSolid);
+			XSetTSOrigin(g_display, g_gc, 0, 0);
+			ui_destroy_glyph((HGLYPH) fill);
+			break;
+
+		case 3:	/* Pattern */
+			for (i = 0; i != 8; i++)
+				ipattern[7 - i] = brush->pattern[i];
+			fill = (Pixmap) ui_create_glyph(8, 8, ipattern);
+			SET_FOREGROUND(bgcolour);
+			SET_BACKGROUND(fgcolour);
+			XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
+			XSetStipple(g_display, g_gc, fill);
+			XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
+			DRAW_ELLIPSE(x, y, cx, cy, fillmode);
+			XSetFillStyle(g_display, g_gc, FillSolid);
+			XSetTSOrigin(g_display, g_gc, 0, 0);
+			ui_destroy_glyph((HGLYPH) fill);
+			break;
+
+		default:
+			unimpl("brush %d\n", brush->style);
+	}
+
+	RESET_FUNCTION(opcode);
+}
+
 /* warning, this function only draws on wnd or backstore, not both */
 void
 ui_draw_glyph(int mixmode,
@@ -2221,11 +2401,13 @@ ui_draw_glyph(int mixmode,
 }
 
 void
-ui_draw_text(uint8 font, uint8 flags, int mixmode, int x, int y,
+ui_draw_text(uint8 font, uint8 flags, uint8 opcode, int mixmode, int x, int y,
 	     int clipx, int clipy, int clipcx, int clipcy,
-	     int boxx, int boxy, int boxcx, int boxcy, int bgcolour,
-	     int fgcolour, uint8 * text, uint8 length)
+	     int boxx, int boxy, int boxcx, int boxcy, BRUSH * brush,
+	     int bgcolour, int fgcolour, uint8 * text, uint8 length)
 {
+	/* TODO: use brush appropriately */
+
 	FONTGLYPH *glyph;
 	int i, j, xyoffset, x1, y1;
 	DATABLOB *entry;
